@@ -17,7 +17,7 @@
 #' \url{https://support.bioconductor.org}
 #'
 #' The code can be viewed at the GitHub repository,
-#' which also lists the contributtor code of conduct:
+#' which also lists the contributor code of conduct:
 #'
 #' \url{https://github.com/mikelove/tximport}
 #' 
@@ -102,7 +102,9 @@ NULL
 #' \code{whitelist.txt};
 #' \code{tierImport} (FALSE) import the tier information in addition to counts;
 #' \code{forceSlow} (FALSE) force the use of the slower import R code
-#' even if \code{fishpond} is installed.
+#' even if \code{fishpond} is installed;
+#' \code{dropMeanVar} (FALSE) don't import inferential mean and variance
+#' matrices even if they exist (also skips inferential replicates)
 #' For \code{type="alevin"} all arguments other than \code{files},
 #' \code{dropInfReps}, and \code{alevinArgs} are ignored.
 #' Note that \code{files} should point to a single \code{quants_mat.gz} file,
@@ -191,7 +193,8 @@ NULL
 #' The formula used to calculate counts is:
 #' \code{cov * transcript length / read length}
 #' @param alevinArgs named list, with logical elements \code{filterBarcodes},
-#' \code{tierImport}, \code{forceSlow}. See Details for definitions.
+#' \code{tierImport}, \code{forceSlow}, \code{dropMeanVar}.
+#' See Details for definitions.
 #' 
 #' @return A simple list containing matrices: abundance, counts, length.
 #' Another list element 'countsFromAbundance' carries through
@@ -207,8 +210,7 @@ NULL
 #' If \code{varReduce=TRUE} the inferential replicates will be summarized
 #' according to the sample variance, and stored as a matrix \code{variance}.
 #' alevin already computes the variance of the bootstrap inferential replicates
-#' and so this is imported without needing to specify \code{varReduce=TRUE}
-#' (note that alevin uses the 1/N variance estimator, so not the same as \code{var}).
+#' and so this is imported without needing to specify \code{varReduce=TRUE}.
 #' 
 #' @references
 #'
@@ -295,55 +297,40 @@ tximport <- function(files,
   
   # special alevin code
   if (type=="alevin") {
-    
     # unpacking alevinArgs
     if (is.null(alevinArgs)) {
-      alevinArgs <- list(filterBarcodes=FALSE,
-                         tierImport=FALSE,
-                         forceSlow=FALSE)
+      alevinArgs <- list(filterBarcodes=FALSE, tierImport=FALSE,
+                         forceSlow=FALSE, dropMeanVar=FALSE)
     }
     stopifnot(is(alevinArgs, "list"))
     stopifnot(all(sapply(alevinArgs, is.logical)))
-    alevinArgNms <- c("filterBarcodes","tierImport","forceSlow")
+    alevinArgNms <- c("filterBarcodes","tierImport","forceSlow","dropMeanVar")
     stopifnot(all(names(alevinArgs) %in% alevinArgNms))
     filterBarcodes <- if (is.null(alevinArgs$filterBarcodes)) FALSE else alevinArgs$filterBarcodes
     tierImport <- if (is.null(alevinArgs$tierImport)) FALSE else alevinArgs$tierImport
     forceSlow <- if (is.null(alevinArgs$forceSlow)) FALSE else alevinArgs$forceSlow
-    
+    dropMeanVar <- if (is.null(alevinArgs$dropMeanVar)) FALSE else alevinArgs$dropMeanVar    
     if (length(files) > 1) stop("alevin import currently only supports a single experiment")
-    vrsn <- getAlevinVersion(files)
-    compareToV014 <- compareVersion(vrsn, "0.14.0")
-    if (compareToV014 == -1) {
-      mat <- readAlevinPreV014(files, filterBarcodes)
-    } else {
-      mat <- readAlevin(files, dropInfReps, filterBarcodes, tierImport, forceSlow)
+    # check that alevin is >= version 0.14.0
+    if (compareVersion(getAlevinVersion(files), "0.14.0") == -1) {
+      stop("use of tximport version >= 1.18 requires alevin version >= 0.14")
     }
-    # lots of if/else based on what we want to return...
+    mat <- readAlevin(files, dropInfReps, filterBarcodes, tierImport, forceSlow, dropMeanVar)
+    # if 'mat' is not a list, it is a matrix with the counts
     if (!is.list(mat)) {
-      txi <- list(abundance=NULL, counts=mat, length=NULL, countsFromAbundance="no")
+      txi <- list(abundance=NULL, counts=mat)
     } else {
+      # otherwise, 'mat' is a list with various information to pass on
+      txi <- list(abundance=NULL, counts=mat$counts, mean=mat$mean, variance=mat$variance)
       if (tierImport) {
-        if ("infReps" %in% names(mat)) {
-          txi <- list(
-            abundance=NULL, counts=mat$counts, tier=mat$tier, mean=mat$mean, variance=mat$variance,
-            infReps=mat$infReps, length=NULL, countsFromAbundance="no")
-        } else {
-          txi <- list(
-            abundance=NULL, counts=mat$counts, tier=mat$tier, mean=mat$mean, variance=mat$variance,
-            length=NULL, countsFromAbundance="no")
-        }
-      } else {
-        if ("infReps" %in% names(mat)) {
-          txi <- list(
-            abundance=NULL, counts=mat$counts, mean=mat$mean, variance=mat$variance,
-            infReps=mat$infReps, length=NULL, countsFromAbundance="no")
-        } else {
-          txi <- list(
-            abundance=NULL, counts=mat$counts, mean=mat$mean, variance=mat$variance,
-            length=NULL, countsFromAbundance="no")
-        }
+        txi$tier <- mat$tier
+      }
+      if ("infReps" %in% names(mat)) {
+        txi$infReps <- mat$infReps
       }
     }
+    txi$length <- NULL
+    txi$countsFromAbundance="no"
     return(txi)
   }
   
@@ -550,13 +537,13 @@ txOut=TRUE, CFA either 'no' or 'scaledTPM', and no inferential replicates")
     countsMatTx <- Matrix::sparseMatrix(i=unlist(countsListI),
                                         j=rep(seq_along(numNonzero), numNonzero),
                                         x=unlist(countsListX),
-                                        dims=c(length(txId),length(files)),
+                                        dims=c(length(txId), length(files)),
                                         dimnames=list(txId, names(files)))
     if (countsFromAbundance == "scaledTPM") {
       abundanceMatTx <- Matrix::sparseMatrix(i=unlist(countsListI),
                                              j=rep(seq_along(numNonzero), numNonzero),
                                              x=unlist(abundanceListX),
-                                             dims=c(length(txId),length(files)),
+                                             dims=c(length(txId), length(files)),
                                              dimnames=list(txId, names(files)))
     } else {
       abundanceMatTx <- NULL
